﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace JonsonsMethod
{
    class Program
    {
        static void Main()
        {
            // Чтение всех строк из файла, где указаны задания и их время выполнения на каждом станке
            string[] lines = File.ReadAllLines("C:\\Users\\admin\\source\\repos\\JonsonsMethod\\JonsonsMethod\\test.txt");
            int n = lines.Length;
            int[,] jobs = new int[n, 2];

            // Заполнение массива jobs значениями из файла
            for (int i = 0; i < n; i++)
            {
                string[] parts = lines[i].Split();
                jobs[i, 0] = int.Parse(parts[0]);
                jobs[i, 1] = int.Parse(parts[1]);
            }

            // Создание графа для исходной матрицы заданий
            List<List<double>> originalGraph = new List<List<double>>();
            for (int i = 0; i < n; i++)
            {
                List<double> job = new List<double>
                {
                    jobs[i, 0],
                    jobs[i, 1]
                };
                originalGraph.Add(job);
            }

            // Вычисление максимального времени простоя для исходной последовательности заданий
            double originalMaxDowntime = Optimality(originalGraph);

            // Применение метода Джонсона для получения оптимального порядка выполнения заданий
            int[] order = Johnson(jobs);

            // Создание графа для оптимальной матрицы заданий
            List<List<double>> graph = new List<List<double>>();
            for (int i = 0; i < n; i++)
            {
                List<double> job = new List<double>
                {
                    jobs[order[i], 0],
                    jobs[order[i], 1]
                };
                graph.Add(job);
            }

            // Вычисление максимального времени простоя для оптимальной последовательности заданий
            double maxDowntime = Optimality(graph);

            // Запись результатов в файл result.txt
            using (StreamWriter sw = new StreamWriter("result.txt"))
            {
                sw.WriteLine($"Время простоя для исходной матрицы: {originalMaxDowntime}");
                for (int i = 0; i < n; i++)
                {
                    sw.WriteLine($"{jobs[i, 0]} {jobs[i, 1]}");
                }
                sw.WriteLine($"Время простоя для изменённой матрицы: {maxDowntime}");
                for (int i = 0; i < n; i++)
                {
                    sw.WriteLine($"{jobs[order[i], 0]} {jobs[order[i], 1]}");
                }
            }
        }

        // Метод Джонсона для нахождения оптимального порядка выполнения заданий
        static int[] Johnson(int[,] jobs)
        {
            int n = jobs.GetLength(0);
            int[] order = new int[n];

            // Инициализация массива order
            for (int i = 0; i < n; i++)
            {
                order[i] = i;
            }

            // Сортировка заданий по возрастанию минимального времени выполнения на любом из станков
            Array.Sort(order, (a, b) =>
                Math.Min(jobs[a, 0], jobs[a, 1]) -
                Math.Min(jobs[b, 0], jobs[b, 1]));

            int[] permutedOrder = new int[n];
            int lowIndex = 0, highIndex = n - 1;

            // Размещение заданий в оптимальном порядке согласно методу Джонсона
            for (int k = 0; k < n; k++)
            {
                if (jobs[order[k], 0] < jobs[order[k], 1])
                {
                    permutedOrder[lowIndex++] = order[k];
                }
                else
                {
                    permutedOrder[highIndex--] = order[k];
                }
            }

            return permutedOrder;
        }

        // Метод для вычисления максимального времени простоя для заданной последовательности заданий
        private static double Optimality(List<List<double>> graph)
        {
            double maxDowntime = graph[0][0];
            double downtime;
            int index = 1;

            while (index < graph.Count)
            {
                double firstSum = 0;
                for (int i = 0; i <= index; i++)
                {
                    firstSum += graph[i][0];
                }

                double secondSum = 0;
                for (int i = 0; i < index; i++)
                {
                    secondSum += graph[i][1];
                }

                downtime = firstSum - secondSum;
                if (maxDowntime < downtime)
                {
                    maxDowntime = downtime;
                }

                index++;
            }

            return maxDowntime;
        }
    }
}
